@@ -105,6 +105,7 @@ class FieldSchema:
         _dict["description"] = self.description
         _dict["type"] = self.type
         _dict["params"] = self.params or dict()
+        _dict["is_primary"] = self.is_primary_key
         return _dict
 
 
@@ -234,7 +235,7 @@ class Hits(LoopBase):
     def get__item(self, item):
         entity_id = self._raw.IDs[item]
         if self._raw.row_data and len(self._raw.row_data) > item:
-           entity_row_data = self._raw.row_data[item]
+            entity_row_data = self._raw.row_data[item]
         else:
             entity_row_data = None
         entity_score = self._raw.scores[item]
@@ -277,6 +278,34 @@ class QueryResult(LoopBase):
         return Hits(self._hits[item], self._auto_id)
 
 
+class ChunkedQueryResult(LoopBase):
+    def __init__(self, raw_list, auto_id=True):
+        super().__init__()
+        self._raw_list = raw_list
+        self._auto_id = auto_id
+
+        self._pack(self._raw_list)
+
+    def __len__(self):
+        return self._nq
+
+    def __len(self):
+        return self._nq
+
+    def _pack(self, raw_list):
+        self._hits = []
+        for raw in raw_list:
+            for bs in raw.hits:
+                hit = milvus_pb2.Hits()
+                hit.ParseFromString(bs)  # otherwise we can use map to simplify this
+                self._hits.append(hit)
+        self._nq = len(self._hits)
+        self._topk = len(self._hits[0].IDs) if len(self._hits) > 0 else 0
+
+    def get__item(self, item):
+        return Hits(self._hits[item], self._auto_id)
+
+
 def _abstract():
     raise NotImplementedError('You need to override this function')
 
@@ -619,17 +648,13 @@ class ConnectIntf:
         _abstract()
 
     def load_collection(self, collection_name, timeout):
-
         _abstract()
 
     def release_collection(self, collection_name, timeout):
-
         _abstract()
 
     def load_partitions(self, collection_name, timeout):
-
         _abstract()
 
     def release_partitions(self, collection_name, timeout):
-
         _abstract()
