@@ -4,6 +4,7 @@ from urllib.parse import urlparse
 import logging
 import threading
 import json
+import functools
 
 import grpc
 from grpc._cython import cygrpc
@@ -11,9 +12,9 @@ from grpc._cython import cygrpc
 from ..grpc_gen import common_pb2 as common_types
 from ..grpc_gen import milvus_pb2_grpc
 from ..grpc_gen import milvus_pb2 as milvus_types
-from .abstract import QueryResult, CollectionSchema
+from .abstract import QueryResult, CollectionSchema, ChunkedQueryResult
 from .prepare import Prepare
-from .types import Status, IndexState, SegmentState
+from .types import Status, IndexState, SegmentState, DataType, DeployMode, ErrorCode
 from .check import (
     int_or_str,
     is_legal_host,
@@ -21,7 +22,16 @@ from .check import (
 )
 
 from .abs_client import AbsMilvus
-from .asynch import SearchFuture, InsertFuture, CreateIndexFuture, FlushFuture, LoadCollectionFuture, LoadPartitionsFuture
+from .asynch import (
+    SearchFuture,
+    InsertFuture,
+    CreateIndexFuture,
+    CreateFlatIndexFuture,
+    FlushFuture,
+    LoadCollectionFuture,
+    LoadPartitionsFuture,
+    ChunkedSearchFuture
+)
 
 from .hooks import BaseSearchHook
 from .client_hooks import SearchHook, HybridSearchHook
@@ -50,7 +60,6 @@ def error_handler(*rargs):
                     raise CollectionNotExistException(e.code, e.message)
 
                 raise e
-
             except grpc.FutureTimeoutError as e:
                 record_dict["RPC timeout"] = str(datetime.datetime.now())
                 LOGGER.error("\nAddr [{}] {}\nRequest timeout: {}\n\t{}".format(self.server_address, func.__name__, e,
@@ -72,6 +81,18 @@ def error_handler(*rargs):
     return wrapper
 
 
+def check_has_collection(func):
+    @functools.wraps(func)
+    def handler(self, *args, **kwargs):
+        collection_name = args[0]
+        if not self.has_collection(collection_name):
+            raise CollectionNotExistException(ErrorCode.CollectionNotExists,
+                                              f"collection {collection_name} doesn't exist!")
+        return func(self, *args, **kwargs)
+
+    return handler
+
+
 def set_uri(host, port, uri):
     if host is not None:
         _port = port if port is not None else config.GRPC_PORT
@@ -332,8 +353,8 @@ class GrpcHandler(AbsMilvus):
         return self._uri
 
     @error_handler()
-    def create_collection(self, collection_name, fields, timeout=30):
-        request = Prepare.create_collection_request(collection_name, fields)
+    def create_collection(self, collection_name, fields, timeout=None, **kwargs):
+        request = Prepare.create_collection_request(collection_name, fields, **kwargs)
 
         rf = self._stub.CreateCollection.future(request, wait_for_ready=True, timeout=timeout)
         status = rf.result()
@@ -346,7 +367,7 @@ class GrpcHandler(AbsMilvus):
         # return Status(status.error_code, status.reason)
 
     @error_handler()
-    def drop_collection(self, collection_name, timeout=20):
+    def drop_collection(self, collection_name, timeout=None):
         request = Prepare.drop_collection_request(collection_name)
 
         rf = self._stub.DropCollection.future(request, wait_for_ready=True, timeout=timeout)
@@ -357,7 +378,7 @@ class GrpcHandler(AbsMilvus):
         # return Status(status.error_code, status.reason)
 
     @error_handler(False)
-    def has_collection(self, collection_name, timeout=30, **kwargs):
+    def has_collection(self, collection_name, timeout=None, **kwargs):
         request = Prepare.has_collection_request(collection_name)
 
         rf = self._stub.HasCollection.future(request, wait_for_ready=True, timeout=timeout)
@@ -369,7 +390,7 @@ class GrpcHandler(AbsMilvus):
         raise BaseException(reply.status.error_code, reply.status.reason)
 
     @error_handler(None)
-    def describe_collection(self, collection_name, timeout=30, **kwargs):
+    def describe_collection(self, collection_name, timeout=None, **kwargs):
         request = Prepare.describe_collection_request(collection_name)
         rf = self._stub.DescribeCollection.future(request, wait_for_ready=True, timeout=timeout)
         response = rf.result()
@@ -384,7 +405,7 @@ class GrpcHandler(AbsMilvus):
         # return Status(status.error_code, status.reason), CollectionSchema(None)
 
     @error_handler([])
-    def list_collections(self, timeout=30):
+    def list_collections(self, timeout=None):
         request = Prepare.show_collections_request()
         rf = self._stub.ShowCollections.future(request, wait_for_ready=True, timeout=timeout)
         response = rf.result()
@@ -395,7 +416,7 @@ class GrpcHandler(AbsMilvus):
         raise BaseException(status.error_code, status.reason)
 
     @error_handler()
-    def create_partition(self, collection_name, partition_tag, timeout=30):
+    def create_partition(self, collection_name, partition_tag, timeout=None):
         request = Prepare.create_partition_request(collection_name, partition_tag)
         rf = self._stub.CreatePartition.future(request, wait_for_ready=True, timeout=timeout)
         response = rf.result()
@@ -405,7 +426,7 @@ class GrpcHandler(AbsMilvus):
         # return Status(response.status.error_code, response.status.reason)
 
     @error_handler()
-    def drop_partition(self, collection_name, partition_tag, timeout=30):
+    def drop_partition(self, collection_name, partition_tag, timeout=None):
         request = Prepare.drop_partition_request(collection_name, partition_tag)
 
         rf = self._stub.DropPartition.future(request, wait_for_ready=True, timeout=timeout)
@@ -417,7 +438,7 @@ class GrpcHandler(AbsMilvus):
         # return Status(response.error_code, response.reason)
 
     @error_handler(False)
-    def has_partition(self, collection_name, partition_tag, timeout=30):
+    def has_partition(self, collection_name, partition_tag, timeout=None):
         request = Prepare.has_partition_request(collection_name, partition_tag)
         rf = self._stub.HasPartition.future(request, wait_for_ready=True, timeout=timeout)
         response = rf.result()
@@ -428,7 +449,7 @@ class GrpcHandler(AbsMilvus):
         raise BaseException(status.error_code, status.reason)
 
     @error_handler(None)
-    def get_partition_info(self, collection_name, partition_tag, timeout=30):
+    def get_partition_info(self, collection_name, partition_tag, timeout=None):
         request = Prepare.partition_stats_request(collection_name, partition_tag)
         rf = self._stub.DescribePartition.future(request, wait_for_ready=True, timeout=timeout)
         response = rf.result()
@@ -442,7 +463,7 @@ class GrpcHandler(AbsMilvus):
         raise BaseException(status.error_code, status.reason)
 
     @error_handler([])
-    def list_partitions(self, collection_name, timeout=30):
+    def list_partitions(self, collection_name, timeout=None):
         request = Prepare.show_partitions_request(collection_name)
 
         rf = self._stub.ShowPartitions.future(request, wait_for_ready=True, timeout=timeout)
@@ -453,6 +474,17 @@ class GrpcHandler(AbsMilvus):
 
         raise BaseException(status.error_code, status.reason)
 
+    @error_handler(None)
+    def get_partition_stats(self, collection_name, partition_name, timeout=None, **kwargs):
+        index_param = Prepare.get_partition_stats_request(collection_name, partition_name)
+        future = self._stub.GetPartitionStatistics.future(index_param, wait_for_ready=True, timeout=timeout)
+        response = future.result()
+        status = response.status
+        if status.error_code == 0:
+            return response.stats
+
+        raise BaseException(status.error_code, status.reason)
+
     # https://github.com/zilliztech/milvus-distributed/issues/1339
     def _prepare_bulk_insert_request(self, collection_name, entities, ids=None, partition_tag=None, params=None,
                                      timeout=None, **kwargs):
@@ -481,7 +513,8 @@ class GrpcHandler(AbsMilvus):
         fields_info = collection_schema["fields"]
 
         if (auto_id and len(entities) != len(fields_info)) \
-                or (not auto_id and ids is not None and len(entities) == len(fields_info)):
+                or (not auto_id and ids is not None and len(entities) == len(fields_info)
+                    and not kwargs.get("orm", False)):
             raise ParamError("The length of entities must be equal to the number of fields!")
 
         if ids is None:
@@ -499,7 +532,7 @@ class GrpcHandler(AbsMilvus):
         return request, auto_id
 
     @error_handler([])
-    def bulk_insert(self, collection_name, entities, ids=None, partition_tag=None, params=None, timeout=30, **kwargs):
+    def bulk_insert(self, collection_name, entities, ids=None, partition_tag=None, params=None, timeout=None, **kwargs):
         try:
             request, auto_id = self._prepare_bulk_insert_request(collection_name, entities, ids, partition_tag, params,
                                                                  timeout, **kwargs)
@@ -525,7 +558,7 @@ class GrpcHandler(AbsMilvus):
             raise err
 
     @error_handler([])
-    def insert(self, collection_name, entities, ids=None, partition_tag=None, params=None, timeout=30, **kwargs):
+    def insert(self, collection_name, entities, ids=None, partition_tag=None, params=None, timeout=None, **kwargs):
         insert_param = kwargs.get('insert_param', None)
 
         # if insert_param and not isinstance(insert_param, grpc_types.InsertParam):
@@ -585,57 +618,97 @@ class GrpcHandler(AbsMilvus):
 
         raise BaseException(response.status.error_code, response.status.reason)
 
-    def _prepare_search_request(self, collection_name, query_entities, partition_tags=None, fields=None, **kwargs):
-        to = kwargs.get("timeout", None)
+    def _prepare_search_request(self, collection_name, query_entities, partition_tags=None, fields=None, timeout=None, **kwargs):
         rf = self._stub.HasCollection.future(Prepare.has_collection_request(collection_name), wait_for_ready=True,
-                                             timeout=to)
+                                             timeout=timeout)
         reply = rf.result()
         if reply.status.error_code != 0 or not reply.value:
             raise CollectionNotExistException(reply.status.error_code, "collection not exists")
 
         request = Prepare.search_request(collection_name, query_entities, partition_tags, fields)
-        collection_schema = self.describe_collection(collection_name=collection_name)
+        collection_schema = self.describe_collection(collection_name, timeout)
         auto_id = collection_schema["auto_id"]
 
         return request, auto_id
 
+    def _divide_search_request(self, collection_name, query_entities, partition_tags=None, fields=None, timeout=None, **kwargs):
+        rf = self._stub.HasCollection.future(Prepare.has_collection_request(collection_name), wait_for_ready=True,
+                                             timeout=timeout)
+        reply = rf.result()
+        if reply.status.error_code != 0 or not reply.value:
+            raise CollectionNotExistException(reply.status.error_code, "collection not exists")
+
+        requests = Prepare.divide_search_request(collection_name, query_entities, partition_tags, fields)
+        collection_schema = self.describe_collection(collection_name, timeout)
+        auto_id = collection_schema["auto_id"]
+
+        return requests, auto_id
+
     @error_handler(None)
-    def search(self, collection_name, query_entities, partition_tags=None, fields=None, **kwargs):
-        to = kwargs.get("timeout", None)
+    def _execute_search_requests(self, requests, timeout=None, **kwargs):
+        auto_id = kwargs.get("auto_id", True)
 
         try:
-            request, auto_id = self._prepare_search_request(collection_name, query_entities, partition_tags,
-                                                            fields=None, **kwargs)
+            raws = []
+            futures = []
 
-            self._search_hook.pre_search()
+            # step 1: get future object
+            for request in requests:
+                self._search_hook.pre_search()
 
-            ft = self._stub.Search.future(request, wait_for_ready=True, timeout=to)
-            if kwargs.get("_async", False) is True:
+                ft = self._stub.Search.future(request, wait_for_ready=True, timeout=timeout)
+                futures.append(ft)
+
+            if kwargs.get("_async", False):
                 func = kwargs.get("_callback", None)
-                return SearchFuture(ft, func, auto_id)
+                return ChunkedSearchFuture(futures, func, auto_id)
 
-            response = ft.result()
-            self._search_hook.aft_search()
+            # step2: get results
+            for ft in futures:
+                response = ft.result()
 
-            if self._search_hook.on_response():
-                return response
+                if response.status.error_code != 0:
+                    raise BaseException(response.status.error_code, response.status.reason)
 
-            if response.status.error_code != 0:
-                raise BaseException(response.status.error_code, response.status.reason)
+                raws.append(response)
 
-            # TODO: handler response
-            # resutls = self._search_hook.handle_response(response)
+            return ChunkedQueryResult(raws, auto_id)
 
-            return QueryResult(response, auto_id)
-        # except (CollectionNotExistException, DescribeCollectionException) as err:
-        #     if kwargs.get("_async", False):
-        #         return SearchFuture(None, None, True, err)
-        #     raise err
         except Exception as pre_err:
             if kwargs.get("_async", False):
                 return SearchFuture(None, None, True, pre_err)
             raise pre_err
 
+    def _batch_search(self, collection_name, query_entities, partition_tags=None, fields=None, timeout=None, **kwargs):
+        requests, auto_id = self._divide_search_request(collection_name, query_entities, partition_tags,
+                                                        fields, **kwargs)
+        kwargs["auto_id"] = auto_id
+        return self._execute_search_requests(requests, timeout, **kwargs)
+
+    @error_handler(None)
+    def _total_search(self, collection_name, query_entities, partition_tags=None, fields=None, timeout=None, **kwargs):
+        request, auto_id = self._prepare_search_request(collection_name, query_entities, partition_tags,
+                                                        fields, timeout, **kwargs)
+        kwargs["auto_id"] = auto_id
+        return self._execute_search_requests([request], timeout, **kwargs)
+
+    @error_handler(None)
+    def search(self, collection_name, query_entities, partition_tags=None, fields=None, timeout=None, **kwargs):
+        if kwargs.get("_deploy_mode", DeployMode.Distributed) == DeployMode.StandAlone:
+            return self._total_search(collection_name, query_entities, partition_tags, fields, timeout, **kwargs)
+        return self._batch_search(collection_name, query_entities, partition_tags, fields, timeout, **kwargs)
+
+    @error_handler(None)
+    @check_has_collection
+    def search_with_expression(self, collection_name, data, anns_field, param, limit, expression, partition_names=None,
+                               output_fields=None, timeout=None, **kwargs):
+        requests = Prepare.search_requests_with_expr(collection_name, data, anns_field, param, limit, expression,
+                                                     partition_names, output_fields, **kwargs)
+        collection_schema = self.describe_collection(collection_name, timeout)
+        auto_id = collection_schema["auto_id"]
+        kwargs["auto_id"] = auto_id
+        return self._execute_search_requests(requests, timeout, **kwargs)
+
     @error_handler(None)
     def get_query_segment_infos(self, collection_name, timeout=30, **kwargs):
         req = Prepare.get_query_segment_info_request(collection_name)
@@ -647,7 +720,7 @@ class GrpcHandler(AbsMilvus):
         raise BaseException(status.error_code, status.reason)
 
     @error_handler()
-    def wait_for_load_index_done(self, collection_name, timeout=30, **kwargs):
+    def wait_for_load_index_done(self, collection_name, timeout=None, **kwargs):
         def load_index_done():
             query_segment_infos = self.get_query_segment_infos(collection_name)
             persistent_segment_infos = self.get_persistent_segment_infos(collection_name)
@@ -666,18 +739,48 @@ class GrpcHandler(AbsMilvus):
             if query_segment_ids != persistent_segment_ids:
                 return False
 
-            filtered_query_segment_info = list(filter(lambda info : info.index_name == "_default_idx", query_segment_infos))
-            filtered_query_segment_index_ids = list(map(lambda info : info.indexID, filtered_query_segment_info))
+            filtered_query_segment_info = list(
+                filter(lambda info: info.index_name == "_default_idx", query_segment_infos))
+            filtered_query_segment_index_ids = list(map(lambda info: info.indexID, filtered_query_segment_info))
             return len(set(filtered_query_segment_index_ids)) == 1
 
         while True:
             time.sleep(0.5)
-            print("\n***********  waiting for load index done ***********\n")
             if load_index_done():
                 return
 
     @error_handler()
     def create_index(self, collection_name, field_name, params, timeout=None, **kwargs):
+        index_type = params["index_type"].upper()
+        if index_type == "FLAT":
+            try:
+                collection_desc = self.describe_collection(collection_name, timeout=timeout, **kwargs)
+                valid_field = False
+                for fields in collection_desc["fields"]:
+                    if field_name != fields["name"]:
+                        continue
+                    if fields["type"] != DataType.FLOAT_VECTOR and fields["type"] != DataType.BINARY_VECTOR:
+                        # TODO: add new error type
+                        raise BaseException(Status.UNEXPECTED_ERROR,
+                                            "cannot create index on non-vector field: " + field_name)
+                    valid_field = True
+                    break
+                if not valid_field:
+                    # TODO: add new error type
+                    raise BaseException(Status.UNEXPECTED_ERROR,
+                                        "cannot create index on non-existed field: " + field_name)
+                index_desc = self.describe_index(collection_name, "", timeout=timeout, **kwargs)
+                if index_desc is not None:
+                    self.drop_index(collection_name, field_name, "_default_idx", timeout=timeout, **kwargs)
+                res_status = Status(Status.SUCCESS, "Warning: It is not necessary to build index with index_type: FLAT")
+                if kwargs.get("_async", False):
+                    return CreateFlatIndexFuture(res_status)
+                return res_status
+            except Exception as err:
+                if kwargs.get("_async", False):
+                    return CreateFlatIndexFuture(None, None, err)
+                raise err
+
         # sync flush
         _async = kwargs.get("_async", False)
         kwargs["_async"] = False
@@ -711,14 +814,15 @@ class GrpcHandler(AbsMilvus):
         return Status(status.error_code, status.reason)
 
     @error_handler(None)
-    def describe_index(self, collection_name, field_name, timeout=None, **kwargs):
-        request = Prepare.describe_index_request(collection_name, field_name)
+    def describe_index(self, collection_name, index_name, timeout=None, **kwargs):
+        request = Prepare.describe_index_request(collection_name, index_name)
 
         rf = self._stub.DescribeIndex.future(request, wait_for_ready=True, timeout=timeout)
         response = rf.result()
         status = response.status
         if status.error_code == 0:
             info_dict = {kv.key: kv.value for kv in response.index_descriptions[0].params}
+            info_dict['field_name'] = response.index_descriptions[0].field_name
             if info_dict.get("params", None):
                 info_dict["params"] = json.loads(info_dict["params"])
             return info_dict
@@ -727,7 +831,17 @@ class GrpcHandler(AbsMilvus):
         raise BaseException(status.error_code, status.reason)
 
     @error_handler(IndexState.Failed)
-    def get_index_state(self, collection_name, field_name, timeout=None, **kwargs):
+    def get_index_build_progress(self, collection_name, index_name, timeout=None):
+        request = Prepare.get_index_build_progress(collection_name, index_name)
+        rf = self._stub.GetIndexBuildProgress.future(request, wait_for_ready=True, timeout=timeout)
+        response = rf.result()
+        status = response.status
+        if status.error_code == 0:
+            return {'total_rows':response.total_rows,'indexed_rows':response.indexed_rows}
+        raise BaseException(status.error_code, status.reason)
+
+    @error_handler(IndexState.Failed)
+    def get_index_state(self, collection_name, field_name, timeout=None):
         request = Prepare.get_index_state_request(collection_name, field_name)
         rf = self._stub.GetIndexState.future(request, wait_for_ready=True, timeout=timeout)
         response = rf.result()
@@ -737,17 +851,13 @@ class GrpcHandler(AbsMilvus):
         raise BaseException(status.error_code, status.reason)
 
     @error_handler(False)
-    def wait_for_creating_index(self, collection_name, field_name, timeout=30, **kwargs):
+    def wait_for_creating_index(self, collection_name, field_name, timeout=None):
         while True:
             time.sleep(0.5)
-            print("\n***********  waiting for building index ***********\n")
-            try:
-                state = self.get_index_state(collection_name, field_name, timeout, **kwargs)
-                if state == IndexState.Finished:
-                    return True
-                if state == IndexState.Failed:
-                    return False
-            except BaseException:
+            state = self.get_index_state(collection_name, field_name, timeout)
+            if state == IndexState.Finished:
+                return True
+            if state == IndexState.Failed:
                 return False
 
     @error_handler()
@@ -762,17 +872,29 @@ class GrpcHandler(AbsMilvus):
             self.wait_for_loading_collection(collection_name)
 
     @error_handler()
-    def wait_for_loading_collection(self, collection_name, timeout=30):
+    def load_collection_progress(self, collection_name, timeout=None):
         """
         Block until load collection complete.
         """
 
+        loaded_segments_nums = sum(info.num_rows for info in
+                self.get_query_segment_infos(collection_name, timeout))
+
+        total_segments_nums = sum(info.num_rows for info in
+                                self.get_persistent_segment_infos(collection_name, timeout))
+
+        return {'num_loaded_entities':loaded_segments_nums, 'num_total_entities': total_segments_nums}
+
+
+    @error_handler()
+    def wait_for_loading_collection(self, collection_name, timeout=None):
+        """
+        Block until load collection complete.
+        """
         unloaded_segments = {info.segmentID: info.num_rows for info in
-                                self.get_persistent_segment_infos(collection_name, timeout)}
-        print("unloaded segment numbers: {}".format(len(unloaded_segments)))
+                             self.get_persistent_segment_infos(collection_name, timeout)}
 
         while len(unloaded_segments) > 0:
-            print("\n***********  waiting for load collection ***********\n")
             time.sleep(0.5)
 
             for info in self.get_query_segment_infos(collection_name, timeout):
@@ -780,7 +902,7 @@ class GrpcHandler(AbsMilvus):
                     unloaded_segments.pop(info.segmentID)
 
     @error_handler()
-    def release_collection(self, db_name, collection_name, timeout=30):
+    def release_collection(self, db_name, collection_name, timeout=None):
         request = Prepare.release_collection(db_name, collection_name)
         rf = self._stub.ReleaseCollection.future(request, wait_for_ready=True, timeout=timeout)
         response = rf.result()
@@ -788,7 +910,7 @@ class GrpcHandler(AbsMilvus):
             raise BaseException(response.error_code, response.reason)
 
     @error_handler()
-    def load_partitions(self, db_name, collection_name, partition_names, timeout=30, **kwargs):
+    def load_partitions(self, db_name, collection_name, partition_names, timeout=None, **kwargs):
         request = Prepare.load_partitions(db_name=db_name, collection_name=collection_name,
                                           partition_names=partition_names)
         future = self._stub.LoadPartitions.future(request, wait_for_ready=True, timeout=timeout)
@@ -815,7 +937,7 @@ class GrpcHandler(AbsMilvus):
             self.wait_for_loading_partitions(collection_name, partition_names)
 
     @error_handler()
-    def wait_for_loading_partitions(self, collection_name, partition_names, timeout=30):
+    def wait_for_loading_partitions(self, collection_name, partition_names, timeout=None):
         """
         Block until load partition complete.
         """
@@ -833,18 +955,41 @@ class GrpcHandler(AbsMilvus):
                              self.get_persistent_segment_infos(collection_name, timeout)
                              if info.partitionID in pIDs}
 
-        print("unloaded segment numbers: {}".format(len(unloaded_segments)))
-
         while len(unloaded_segments) > 0:
-            print("\n***********  waiting for load collection ***********\n")
             time.sleep(0.5)
 
             for info in self.get_query_segment_infos(collection_name, timeout):
                 if 0 <= unloaded_segments.get(info.segmentID, -1) <= info.num_rows:
                     unloaded_segments.pop(info.segmentID)
+                    
+    @error_handler()
+    def load_partitions_progress(self, collection_name, partition_names, timeout=None):
+        """
+        Block until load collection complete.
+        """
+        request = Prepare.show_partitions_request(collection_name)
+        rf = self._stub.ShowPartitions.future(request, wait_for_ready=True, timeout=timeout)
+        response = rf.result()
+        status = response.status
+        if status.error_code != 0:
+            raise BaseException(status.error_code, status.reason)
+
+        pIDs = [response.partitionIDs[index] for index, p_name in enumerate(response.partition_names)
+                if p_name in partition_names]
+
+
+        total_segments_nums = sum(info.num_rows for info in
+                             self.get_persistent_segment_infos(collection_name, timeout)
+                             if info.partitionID in pIDs)
+
+        loaded_segments_nums = sum(info.num_rows for info in
+                             self.get_query_segment_infos(collection_name, timeout)
+                             if info.partitionID in pIDs)
+
+        return {'num_loaded_entities':loaded_segments_nums, 'num_total_entities': total_segments_nums}
 
     @error_handler()
-    def release_partitions(self, db_name, collection_name, partition_names, timeout=30):
+    def release_partitions(self, db_name, collection_name, partition_names, timeout=None):
         request = Prepare.release_partitions(db_name, collection_name,
                                              partition_names)
         rf = self._stub.ReleasePartitions.future(request, wait_for_ready=True, timeout=timeout)
@@ -861,8 +1006,10 @@ class GrpcHandler(AbsMilvus):
         if status.error_code == 0:
             return response.stats
 
+        raise BaseException(status.error_code, status.reason)
+
     @error_handler(None)
-    def get_persistent_segment_infos(self, collection_name, timeout=30, **kwargs):
+    def get_persistent_segment_infos(self, collection_name, timeout=None, **kwargs):
         req = Prepare.get_persistent_segment_info_request(collection_name)
         future = self._stub.GetPersistentSegmentInfo.future(req, wait_for_ready=True, timeout=timeout)
         response = future.result()
@@ -872,22 +1019,46 @@ class GrpcHandler(AbsMilvus):
         raise BaseException(status.error_code, status.reason)
 
     @error_handler()
-    def wait_for_flushed(self, collection_name, timeout=30, **kwargs):
-        def flushed():
-            try:
-                infos = self.get_persistent_segment_infos(collection_name, timeout, **kwargs)
-                for info in infos:
-                    print("info.state: ", info.state)
-                    if info.state != SegmentState.Flushed:
-                        return False
-                return True
-            except BaseException:
-                return False
+    def wait_for_flushed(self, collection_name, timeout=None, **kwargs):
+        def get_first_segment_ids():
+            infos = self.get_persistent_segment_infos(collection_name, timeout, **kwargs)
+            return [info.segmentID for info in infos]
+            # in fact, exception only happens when collection is not existed
+            # try:
+            #     infos = self.get_persistent_segment_infos(collection_name, timeout, **kwargs)
+            #     return [info.segmentID for info in infos]
+            # except BaseException:
+            #     return None
 
+        def flushed(segment_ids_to_wait):
+            infos = self.get_persistent_segment_infos(collection_name, timeout, **kwargs)
+            need_cnt = len(segment_ids_to_wait)
+            have_cnt = 0
+            for info in infos:
+                if info.segmentID not in segment_ids_to_wait:
+                    continue
+
+                if info.state == SegmentState.Flushed:
+                    have_cnt += 1
+                    # return False
+            return need_cnt == have_cnt
+            # in fact, exception only happens when collection is not existed
+            # try:
+            #     infos = self.get_persistent_segment_infos(collection_name, timeout, **kwargs)
+            #     for info in infos:
+            #         if info.segmentID not in segment_ids_to_wait:
+            #             continue
+            #         print("info.state: ", info.state)
+            #         if info.state != SegmentState.Flushed:
+            #             return False
+            #     return True
+            # except BaseException:
+            #     return False
+
+        first_segment_ids = get_first_segment_ids()
         while True:
             time.sleep(0.5)
-            print("\n***********  waiting for flushed ***********\n")
-            if flushed():
+            if flushed(first_segment_ids):
                 return
 
     @error_handler()
@@ -919,7 +1090,7 @@ class GrpcHandler(AbsMilvus):
                 self.wait_for_flushed(collection_name)
 
     @error_handler()
-    def drop_index(self, collection_name, field_name, index_name, timeout=30, **kwargs):
+    def drop_index(self, collection_name, field_name, index_name, timeout=None, **kwargs):
         request = Prepare.drop_index_request(collection_name, field_name, index_name)
         future = self._stub.DropIndex.future(request, wait_for_ready=True, timeout=timeout)
         response = future.result()
@@ -927,7 +1098,7 @@ class GrpcHandler(AbsMilvus):
             raise BaseException(response.error_code, response.reason)
 
     @error_handler(False)
-    def fake_register_link(self, timeout=30):
+    def fake_register_link(self, timeout=None):
         request = Prepare.register_link_request()
         future = self._stub.RegisterLink.future(request, wait_for_ready=True, timeout=timeout)
-        return future.result().status.error_code == 0
+        return future.result().status
