@@ -15,13 +15,13 @@ from ..grpc_gen import milvus_pb2 as milvus_types
 
 from . import blob
 
-from .types import RangeType, DataType, MetricType, IndexType, PlaceholderType
+from .types import RangeType, DataType, MetricType, IndexType, PlaceholderType, DeployMode
 
 
 class Prepare:
 
     @classmethod
-    def create_collection_request(cls, collection_name, fields):
+    def create_collection_request(cls, collection_name, fields, **kwargs):
         """
         :type param: dict
         :param param: (Required)
@@ -49,7 +49,7 @@ class Prepare:
         auto_id = fields.get('auto_id', True)
         schema.autoID = auto_id
 
-        if not auto_id:
+        if not auto_id and not kwargs.get("orm", False):
             field_schema = schema_types.FieldSchema()
 
             field_schema.name = "_id"
@@ -71,7 +71,7 @@ class Prepare:
                     raise ParamError("You should specify the name of field!")
                 field_schema.name = field_name
 
-                is_primary_key = field.get('is_primary_key', False)
+                is_primary_key = field.get('is_primary_key', False) or field.get("is_primary", False)
                 if is_primary_key and auto_id:
                     raise ParamError("primary key is not supported when auto_id is True")
                 field_schema.is_primary_key = is_primary_key
@@ -439,6 +439,112 @@ class Prepare:
 
         return row_batch
 
+    @classmethod
+    def _prepare_placeholders(cls, vectors, nq, max_nq_per_batch, tag, pl_type, is_binary):
+        pls = []
+        for begin in range(0, nq, max_nq_per_batch):
+            end = min(begin + max_nq_per_batch, nq)
+            pl = milvus_types.PlaceholderValue(tag=tag)
+            pl.type = pl_type
+            for i in range(begin, end):
+                if is_binary:
+                    pl.values.append(blob.vectorBinaryToBytes(vectors[i]))
+                else:
+                    pl.values.append(blob.vectorFloatToBytes(vectors[i]))
+            pls.append(pl)
+        return pls
+
+    @classmethod
+    def divide_search_request(cls, collection_name, query_entities, partition_tags=None, fields=None, **kwargs):
+        if not isinstance(query_entities, (dict,)):
+            raise ParamError("Invalid query format. 'query_entities' must be a dict")
+
+        duplicated_entities = copy.deepcopy(query_entities)
+        vector_placeholders = dict()
+
+        meta = {}   # TODO: ugly here, find a better method
+        def extract_vectors_param(param, placeholders, meta=None):
+            if not isinstance(param, (dict, list)):
+                return
+
+            if isinstance(param, dict):
+                if "vector" in param:
+                    # TODO: Here may not replace ph
+                    ph = "$" + str(len(placeholders))
+
+                    for pk, pv in param["vector"].items():
+                        if "query" not in pv:
+                            raise ParamError("param vector must contain 'query'")
+                        if "topk" not in pv:
+                            raise ParamError("dsl must contain 'topk'")
+                        topk = pv["topk"]
+                        if not isinstance(topk, (int, str)):
+                            raise ParamError("topk must be int or str")
+                        try:
+                            topk = int(topk)
+                        except Exception:
+                            raise ParamError("topk is not illegal")
+                        if topk < 0:
+                            raise ParamError("topk must be greater than zero")
+                        meta["topk"] = topk
+                        placeholders[ph] = pv["query"]
+                        param["vector"][pk]["query"] = ph
+
+                    return
+                else:
+                    for _, v in param.items():
+                        extract_vectors_param(v, placeholders, meta)
+
+            if isinstance(param, list):
+                for item in param:
+                    extract_vectors_param(item, placeholders, meta)
+
+        extract_vectors_param(duplicated_entities, vector_placeholders, meta)
+
+        if len(vector_placeholders) > 1:
+            raise ParamError("query on two vector field is not supported now!")
+
+        requests = []
+        factor = 10
+        topk = meta.get("topk", 100)    # TODO: ugly here, find a better method
+        for tag, vectors in vector_placeholders.items():
+            if len(vectors) <= 0:
+                continue
+
+            if isinstance(vectors[0], bytes):
+                bytes_per_vector = len(vectors[0])
+            else:
+                bytes_per_vector = len(vectors[0]) * 4
+
+            nq = len(vectors)
+            max_nq_per_batch = (5 * 1024 * 1024) / (bytes_per_vector * topk * factor)
+            max_nq_per_batch = int(max_nq_per_batch)
+            if max_nq_per_batch <= 0:
+                raise ParamError(f"topk {topk} is too large!")
+
+            if isinstance(vectors[0], bytes):
+                is_binary = True
+                pl_type = PlaceholderType.BinaryVector
+            else:
+                is_binary = False
+                pl_type = PlaceholderType.FloatVector
+
+            pls = cls._prepare_placeholders(vectors, nq, max_nq_per_batch, tag, pl_type, is_binary)
+
+            for pl in pls:
+                plg = milvus_types.PlaceholderGroup()
+                plg.placeholders.append(pl)
+                plg_str = milvus_types.PlaceholderGroup.SerializeToString(plg)
+                request = milvus_types.SearchRequest(
+                    collection_name=collection_name,
+                    partition_names=partition_tags,
+                )
+                request.dsl = ujson.dumps(duplicated_entities)
+                request.placeholder_group = plg_str
+                requests.append(request)
+
+            return requests
+
     @classmethod
     def search_request(cls, collection_name, query_entities, partition_tags=None, fields=None, **kwargs):
         if not isinstance(query_entities, (dict,)):
@@ -501,6 +607,61 @@ class Prepare:
 
         return request
 
+    @classmethod
+    def search_requests_with_expr(cls, collection_name, data, anns_field, param, limit, expr, partition_names=None, output_fields=None, **kwargs):
+        requests = []
+
+        if len(data) <= 0:
+            return requests
+
+        if isinstance(data[0], bytes):
+            bytes_per_vector = len(data[0])
+            is_binary = True
+            pl_type = PlaceholderType.BinaryVector
+        else:
+            bytes_per_vector = len(data[0]) * 4
+            is_binary = False
+            pl_type = PlaceholderType.FloatVector
+
+        nq = len(data)
+        max_nq_per_batch = nq
+        if kwargs.get("_deploy_mode", DeployMode.Distributed):
+            factor = 10
+            max_nq_per_batch = (5 * 1024 * 1024) / (bytes_per_vector * limit * factor)
+            max_nq_per_batch = int(max_nq_per_batch)
+            if max_nq_per_batch <= 0:
+                raise ParamError(f"limit {limit} is too large!")
+
+        tag = "$0"
+        pls = cls._prepare_placeholders(data, nq, max_nq_per_batch, tag, pl_type, is_binary)
+
+        metric_type = param.pop("metric_type", "L2")
+        search_params = {"anns_field": anns_field, "topk": limit, "metric_type": metric_type, "params": param.pop("params", None)}
+
+        def dump(v):
+            if isinstance(v, dict):
+                return ujson.dumps(v)
+            return str(v)
+
+        for pl in pls:
+            plg = milvus_types.PlaceholderGroup()
+            plg.placeholders.append(pl)
+            plg_str = milvus_types.PlaceholderGroup.SerializeToString(plg)
+            request = milvus_types.SearchRequest(
+                collection_name=collection_name,
+                partition_names=partition_names,
+            )
+            request.placeholder_group = plg_str
+
+            request.dsl_type = common_types.DslType.BoolExprV1
+            request.dsl = expr
+            request.search_params.extend([common_types.KeyValuePair(key=str(key), value=dump(value))
+                                          for key, value in search_params.items()])
+
+            requests.append(request)
+
+        return requests
+
     @classmethod
     def create_index__request(cls, collection_name, field_name, params):
         index_params = milvus_types.CreateIndexRequest(collection_name=collection_name, field_name=field_name)
@@ -525,13 +686,17 @@ class Prepare:
         return index_params
 
     @classmethod
-    def describe_index_request(cls, collection_name, field_name):
-        return milvus_types.DescribeIndexRequest(collection_name=collection_name, field_name=field_name)
+    def describe_index_request(cls, collection_name, index_name):
+        return milvus_types.DescribeIndexRequest(collection_name=collection_name, index_name=index_name)
 
     @classmethod
     def describe_index_progress_request(cls, collection_name, field_name):
         return milvus_types.DescribeIndexProgressRequest(collection_name=collection_name, field_name=field_name)
 
+    @classmethod
+    def get_index_build_progress(cls, collection_name, field_name):
+        return milvus_types.GetIndexBuildProgressRequest(collection_name=collection_name, field_name=field_name)
+
     @classmethod
     def get_index_state_request(cls, collection_name, field_name):
         return milvus_types.GetIndexStateRequest(collection_name=collection_name, field_name=field_name)
@@ -574,3 +739,8 @@ class Prepare:
     def drop_index_request(cls, collection_name, field_name, index_name):
         return milvus_types.DropIndexRequest(db_name="", collection_name=collection_name, field_name=field_name,
                                              index_name=index_name)
+
+    @classmethod
+    def get_partition_stats_request(cls, collection_name, partition_name):
+        return milvus_types.GetPartitionStatisticsRequest(db_name="", collection_name=collection_name,
+                                                          partition_name=partition_name)
