@@ -8,10 +8,10 @@ import time
 from urllib.parse import urlparse
 
 from . import __version__
-from .types import Status
+from .types import Status, DataType, DeployMode
 from .check import check_pass_param, is_legal_host, is_legal_port, is_legal_index_metric_type, is_legal_binary_index_metric_type
 from .pool import ConnectionPool, SingleConnectionPool, SingletonThreadPool
-from .exceptions import ParamError, DeprecatedError
+from .exceptions import BaseException, ParamError, DeprecatedError
 
 from ..settings import DefaultConfig as config
 from .utils import valid_binary_metric_types
@@ -104,15 +104,17 @@ class Milvus:
         self._kw = kwargs
         self._hooks = collections.defaultdict()
 
+        self._deploy_mode = DeployMode.Distributed
         self._wait_for_healthy()
 
     @check_connect
-    def _wait_for_healthy(self, timeout=30, retry=5):
+    def _wait_for_healthy(self, timeout=300, retry=250):
         with self._connection() as handler:
             while retry > 0:
                 try:
-                    healthy = handler.fake_register_link(timeout)
-                    if healthy:
+                    status = handler.fake_register_link(timeout)
+                    if status.error_code == 0:
+                        self._deploy_mode = status.reason
                         return
                 except:
                     pass
@@ -153,7 +155,7 @@ class Milvus:
         raise Exception("connection was already closed!")
 
     @check_connect
-    def create_collection(self, collection_name, fields, timeout=30):
+    def create_collection(self, collection_name, fields, timeout=None, **kwargs):
         """
         Creates a collection.
 
@@ -186,10 +188,10 @@ class Milvus:
             BaseException: If the return result from server is not ok
         """
         with self._connection() as handler:
-            return handler.create_collection(collection_name, fields, timeout)
+            return handler.create_collection(collection_name, fields, timeout, **kwargs)
 
     @check_connect
-    def drop_collection(self, collection_name, timeout=30):
+    def drop_collection(self, collection_name, timeout=None):
         """
         Deletes a specified collection.
 
@@ -213,7 +215,7 @@ class Milvus:
             return handler.drop_collection(collection_name, timeout)
 
     @check_connect
-    def has_collection(self, collection_name, timeout=30):
+    def has_collection(self, collection_name, timeout=None):
         """
         Checks whether a specified collection exists.
 
@@ -237,7 +239,7 @@ class Milvus:
             return handler.has_collection(collection_name, timeout)
 
     @check_connect
-    def describe_collection(self, collection_name, timeout=30):
+    def describe_collection(self, collection_name, timeout=None):
         """
         Returns the schema of specified collection.
         Example: {'collection_name': 'create_collection_eXgbpOtn', 'auto_id': True, 'description': '',
@@ -265,7 +267,7 @@ class Milvus:
             return handler.describe_collection(collection_name, timeout)
 
     @check_connect
-    def load_collection(self, collection_name, timeout=30):
+    def load_collection(self, collection_name, timeout=None):
         """
         Loads a specified collection from disk to memory.
 
@@ -289,7 +291,7 @@ class Milvus:
             return handler.load_collection("", collection_name=collection_name, timeout=timeout)
 
     @check_connect
-    def release_collection(self, collection_name, timeout=30):
+    def release_collection(self, collection_name, timeout=None):
         """
         Clear collection data from memory.
 
@@ -313,7 +315,7 @@ class Milvus:
             return handler.release_collection(db_name="", collection_name=collection_name, timeout=timeout)
 
     @check_connect
-    def get_collection_stats(self, collection_name, timeout=30, **kwargs):
+    def get_collection_stats(self, collection_name, timeout=None, **kwargs):
         """
         Returns collection statistics information.
         Example: {"row_count": 10}
@@ -340,7 +342,7 @@ class Milvus:
             return result
 
     @check_connect
-    def list_collections(self, timeout=30):
+    def list_collections(self, timeout=None):
         """
         Returns a list of all collection names.
 
@@ -360,7 +362,7 @@ class Milvus:
             return handler.list_collections(timeout)
 
     @check_connect
-    def create_partition(self, collection_name, partition_tag, timeout=30):
+    def create_partition(self, collection_name, partition_tag, timeout=None):
         """
         Creates a partition in a specified collection. You only need to import the
         parameters of partition_tag to create a partition. A collection cannot hold
@@ -389,7 +391,7 @@ class Milvus:
             return handler.create_partition(collection_name, partition_tag, timeout)
 
     @check_connect
-    def drop_partition(self, collection_name, partition_tag, timeout=30):
+    def drop_partition(self, collection_name, partition_tag, timeout=None):
         """
         Deletes the specified partition in a collection. Note that the default partition
         '_default' is not permitted to delete. When a partition deleted, all data stored in it
@@ -418,7 +420,7 @@ class Milvus:
             return handler.drop_partition(collection_name, partition_tag, timeout)
 
     @check_connect
-    def has_partition(self, collection_name, partition_tag, timeout=30):
+    def has_partition(self, collection_name, partition_tag, timeout=None):
         """
         Checks if a specified partition exists in a collection.
 
@@ -445,7 +447,7 @@ class Milvus:
             return handler.has_partition(collection_name, partition_tag, timeout)
 
     @check_connect
-    def load_partitions(self, collection_name, partition_names, timeout=30):
+    def load_partitions(self, collection_name, partition_names, timeout=None):
         """
         Load specified partitions from disk to memory.
 
@@ -473,7 +475,7 @@ class Milvus:
                                            partition_names=partition_names, timeout=timeout)
 
     @check_connect
-    def release_partitions(self, collection_name, partition_names, timeout=30):
+    def release_partitions(self, collection_name, partition_names, timeout=None):
         """
         Clear partitions data from memory.
 
@@ -501,7 +503,7 @@ class Milvus:
                                               partition_names=partition_names, timeout=timeout)
 
     @check_connect
-    def list_partitions(self, collection_name, timeout=30):
+    def list_partitions(self, collection_name, timeout=None):
         """
         Returns a list of all partition tags in a specified collection.
 
@@ -525,6 +527,37 @@ class Milvus:
         with self._connection() as handler:
             return handler.list_partitions(collection_name, timeout)
 
+    @check_connect
+    def get_partition_stats(self, collection_name, partition_name, timeout=None, **kwargs):
+        """
+        Returns partition statistics information.
+        Example: {"row_count": 10}
+
+        :param collection_name: The name of collection.
+        :type  collection_name: str.
+
+        :param partition_name: The name of partition.
+        :type  partition_name: str.
+
+        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
+                        is set to None, client waits until server response or error occur.
+        :type  timeout: float
+
+        :return: statistics information
+        :rtype: dict
+
+        :raises:
+            RpcError: If gRPC encounter an error
+            ParamError: If parameters are invalid
+            BaseException: If the return result from server is not ok
+        """
+        check_pass_param(collection_name=collection_name)
+        with self._connection() as handler:
+            stats = handler.get_partition_stats(collection_name, partition_name, timeout, **kwargs)
+            result = {stat.key: stat.value for stat in stats}
+            result["row_count"] = int(result["row_count"])
+            return result
+
     # @check_connect
     # def insert(self, collection_name, entities, ids=None, partition_tag=None, params=None, timeout=None, **kwargs):
     #     """
@@ -695,7 +728,7 @@ class Milvus:
             return handler.create_index(collection_name, field_name, params, timeout, **kwargs)
 
     @check_connect
-    def drop_index(self, collection_name, field_name, timeout=30):
+    def drop_index(self, collection_name, field_name, timeout=None):
         """
         Removes the index of a field in a specified collection.
 
@@ -724,7 +757,7 @@ class Milvus:
                                       field_name=field_name, index_name="_default_idx", timeout=timeout)
 
     @check_connect
-    def describe_index(self, collection_name, field_name, timeout=30):
+    def describe_index(self, collection_name, index_name="", timeout=None):
         """
         Returns the schema of index built on specified field.
         Example: {'index_type': 'FLAT', 'metric_type': 'L2', 'params': {'nlist': 128}}
@@ -749,10 +782,10 @@ class Milvus:
         """
         check_pass_param(collection_name=collection_name)
         with self._connection() as handler:
-            return handler.describe_index(collection_name, field_name, timeout)
+            return handler.describe_index(collection_name, index_name, timeout)
 
     @check_connect
-    def insert(self, collection_name, entities, ids=None, partition_tag=None, timeout=30, **kwargs):
+    def insert(self, collection_name, entities, ids=None, partition_tag=None, timeout=None, **kwargs):
         """
         Inserts entities in a specified collection.
 
@@ -940,4 +973,83 @@ class Milvus:
             BaseException: If the return result from server is not ok
         """
         with self._connection() as handler:
+            kwargs["_deploy_mode"] = self._deploy_mode
             return handler.search(collection_name, dsl, partition_tags, fields, timeout=timeout, **kwargs)
+
+    @check_connect
+    def search_with_expression(self, collection_name, data, anns_field, param, limit, expression, partition_names=None,
+                               output_fields=None, timeout=None, **kwargs):
+        """
+        Searches a collection based on the given expression and returns query results.
+
+        :param collection_name: The name of the collection to search.
+        :type  collection_name: str
+        :param data: The vectors of search data, the length of data is number of query (nq), the dim of every vector in
+                     data must be equal to vector field's of collection.
+        :type  data: list[list[float]]
+        :param anns_field: The vector field used to search of collection.
+        :type  anns_field: str
+        :param param: The parameters of search, such as nprobe, etc.
+        :type  param: dict
+        :param limit: The max number of returned record, we also called this parameter as topk.
+        :type  limit: int
+        :param expression: The boolean expression used to filter attribute.
+        :type  expression: str
+        :param partition_names: The names of partitions to search.
+        :type  partition_names: list[str]
+        :param output_fields: The fields to return in the search result, not supported now.
+        :type  output_fields: list[str]
+        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
+                        is set to None, client waits until server response or error occur.
+        :type  timeout: float
+        :param kwargs:
+            * *_async* (``bool``) --
+              Indicate if invoke asynchronously. When value is true, method returns a SearchFuture object;
+              otherwise, method returns results from server.
+            * *_callback* (``function``) --
+              The callback function which is invoked after server response successfully. It only take
+              effect when _async is set to True.
+
+        :return: Query result. QueryResult is iterable and is a 2d-array-like class, the first dimension is
+                 the number of vectors to query (nq), the second dimension is the number of limit(topk).
+        :rtype: QueryResult
+
+        :raises:
+            RpcError: If gRPC encounter an error
+            ParamError: If parameters are invalid
+            BaseException: If the return result from server is not ok
+        """
+        with self._connection() as handler:
+            kwargs["_deploy_mode"] = self._deploy_mode
+            return handler.search_with_expression(collection_name, data, anns_field, param, limit, expression, partition_names, output_fields, timeout, **kwargs)
+
+    @check_connect
+    def load_collection_progress(self, collection_name, timeout=None):
+        with self._connection() as handler:
+            return handler.load_collection_progress(collection_name, timeout=timeout)
+
+    @check_connect
+    def load_partitions_progress(self, collection_name, partition_names, timeout=None):
+        with self._connection() as handler:
+            return handler.load_partitions_progress(collection_name, partition_names, timeout=timeout)
+
+    @check_connect
+    def wait_for_loading_collection_complete(self, collection_name, timeout=None):
+        with self._connection() as handler:
+            return handler.wait_for_loading_collection(collection_name, timeout=timeout)
+
+    @check_connect
+    def wait_for_loading_partitions_complete(self, collection_name, partition_names, timeout=None):
+        with self._connection() as handler:
+            return handler.wait_for_loading_partitions(collection_name, partition_names, timeout=timeout)
+
+    @check_connect
+    def get_index_build_progress(self, collection_name, index_name, timeout=None):
+        with self._connection() as handler:
+            return handler.get_index_build_progress(collection_name, index_name, timeout=timeout)
+
+    @check_connect
+    def wait_for_creating_index(self, collection_name, index_name, timeout=None):
+        with self._connection() as handler:
+            return handler.wait_for_creating_index(collection_name, index_name, timeout=timeout)
+
