@@ -50,15 +50,14 @@ def check_invalid_binary_vector(entities):
             if not isinstance(entity['values'], list) and len(entity['values']) == 0:
                 return False
             else:
-                dim = len(entity['values'][0])
+                dim = len(entity['values'][0]) * 8
                 if dim == 0:
                     return False
                 for values in entity['values']:
-                    if len(values) != dim:
+                    if len(values) * 8 != dim:
+                        return False
+                    if not isinstance(values, bytes):
                         return False
-                    for value in values:
-                        if not isinstance(value, bytes):
-                            return False
     return True
 
 # def merge_results(results_list, topk, *args, **kwargs):
