@@ -1803,6 +1803,81 @@ class TestSearchInvalid(object):
             res = connect.search(collection, query)
 
 
+class TestSearchWithExpr(object):
+    def test_search_with_expression_naive(self, connect):
+        collection = "test_search_with_expression_naive"
+        year_field = "year"
+        vec_field = "vec"
+        dim = 128
+        nb = 3000
+        nq = 10
+        topk = 10
+        expr = "year > 20"
+
+        fields = {
+            "fields": [
+                {"name": year_field, "type": DataType.INT64},
+                {"name": vec_field, "type": DataType.FLOAT_VECTOR,
+                 "params": {"dim": dim},
+                 "indexes": [{"metric_type": "L2"}]},
+            ],
+            "auto_id": True
+        }
+        if connect.has_collection(collection):
+            connect.drop_collection(collection)
+        connect.create_collection(collection, fields)
+
+        vectors = [[random.random() for _ in range(dim)] for _ in range(nb)]
+        vectors = preprocessing.normalize(vectors, axis=1, norm='l2').tolist()
+        entities = [
+            {"name": year_field, "type": DataType.INT64, "values": [i for i in range(nb)]},
+            {"name": vec_field, "type": DataType.FLOAT_VECTOR, "values": vectors}
+        ]
+        connect.insert(collection, entities)
+
+        connect.create_index(collection, vec_field, {"index_type": "IVF_FLAT", "metric_type": "L2", "params": {"nlist": 100}})
+        connect.load_collection(collection)
+
+        data = [[random.random() for _ in range(dim)] for _ in range(nq)]
+        data = preprocessing.normalize(data, axis=1, norm='l2')
+        param = {
+            "metric_type": "L2",
+            "params": {
+                "nprobe": 10,
+            }
+        }
+
+        dsl = {
+            "bool": {
+                "must": [
+                    {
+                        "range": {
+                            year_field: {
+                                "GT": 20
+                            }
+                        }
+                    },
+                    {
+                        "vector": {
+                            vec_field: {
+                                "topk": topk,
+                                "query": data,
+                                "metric_type": "L2",
+                                "params": {
+                                    "nprobe": 10,
+                                }
+                            }
+                        }
+                    }
+                ]
+            }
+        }
+        connect.search(collection, dsl)
+        connect.search_with_expression(collection, data, vec_field, param, topk, expr)
+
+        connect.drop_collection(collection)
+
+
 def check_id_result(result, id):
     limit_in = 5
     ids = [entity.id for entity in result]
